diff -r 7353e3253840 src/imap/imap-client.c
--- a/src/imap/imap-client.c	Wed Dec 08 23:17:11 2010 +0000
+++ b/src/imap/imap-client.c	Thu Dec 09 09:40:22 2010 +0000
@@ -524,6 +524,7 @@
 		cmd->cancel = FALSE;
 		client_send_tagline(cmd, "NO Command cancelled.");
 	}
+	logcs(t_strdup_printf("finished: %s %s", cmd->name, cmd->args));
 
 	if (!cmd->param_error)
 		client->bad_counter = 0;
@@ -664,6 +665,7 @@
 			client_add_missing_io(client);
 			return TRUE;
 		}
+		logcs(t_strdup_printf("unfinished input: %s %s", cmd->name, cmd->args));
 
 		return client_handle_unfinished_cmd(cmd);
 	}
@@ -828,9 +830,10 @@
 	/* continue processing command */
 	finished = cmd->func(cmd) || cmd->state == CLIENT_COMMAND_STATE_DONE;
 
-	if (!finished)
+	if (!finished) {
+		logcs(t_strdup_printf("unfinished output: %s %s", cmd->name, cmd->args));
 		(void)client_handle_unfinished_cmd(cmd);
-	else {
+	} else {
 		/* command execution was finished */
 		client_command_free(&cmd);
 	}
diff -r 7353e3253840 src/imap/imap-sync.c
--- a/src/imap/imap-sync.c	Wed Dec 08 23:17:11 2010 +0000
+++ b/src/imap/imap-sync.c	Thu Dec 09 09:40:22 2010 +0000
@@ -535,6 +535,7 @@
 				client_command_free(&cmd);
 		}
 	}
+	logcs("sync");
 	return cmd_finish_sync(sync_cmd);
 }
 
diff -r 7353e3253840 src/imap/main.c
--- a/src/imap/main.c	Wed Dec 08 23:17:11 2010 +0000
+++ b/src/imap/main.c	Thu Dec 09 09:40:22 2010 +0000
@@ -23,6 +23,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <sys/time.h>
+#include <sys/resource.h>
 #define IS_STANDALONE() \
         (getenv(MASTER_IS_PARENT_ENV) == NULL)
 
@@ -43,6 +45,18 @@
 	return old_hook;
 }
 
+void logcs(const char *name)
+{
+	static struct rusage prev_ru;
+	struct rusage ru;
+
+	if (getrusage(RUSAGE_SELF, &ru) == 0) {
+		i_debug("%s: volcs=%lu (+%u)", name, ru.ru_nvcsw,
+			(unsigned int)(ru.ru_nvcsw - prev_ru.ru_nvcsw));
+		prev_ru = ru;
+	}
+}
+
 void imap_refresh_proctitle(void)
 {
 #define IMAP_PROCTITLE_PREFERRED_LEN 80
@@ -208,6 +222,7 @@
 	if (set->verbose_proctitle)
 		verbose_proctitle = TRUE;
 
+	logcs("client created");
 	client = client_create(fd_in, fd_out, mail_user, user, set);
 	T_BEGIN {
 		client_add_input(client, input_buf);
diff -r 7353e3253840 src/lib-storage/index/maildir/maildir-sync.c
--- a/src/lib-storage/index/maildir/maildir-sync.c	Wed Dec 08 23:17:11 2010 +0000
+++ b/src/lib-storage/index/maildir/maildir-sync.c	Thu Dec 09 09:40:22 2010 +0000
@@ -721,11 +721,13 @@
 	enum maildir_uidlist_sync_flags sync_flags;
 	enum maildir_uidlist_rec_flag flags;
 	bool new_changed, cur_changed, lock_failure;
+	struct rusage ru1, ru2, ru3, ru4, ru5, ru6, ru7;
 	const char *fname;
 	int ret;
 
 	*lost_files_r = FALSE;
 
+	getrusage(RUSAGE_SELF, &ru1);
 	if (forced)
 		new_changed = cur_changed = TRUE;
 	else {
@@ -733,6 +735,7 @@
 		if (ret <= 0)
 			return ret;
 	}
+	getrusage(RUSAGE_SELF, &ru2);
 
 	/*
 	   Locking, locking, locking.. Wasn't maildir supposed to be lockless?
@@ -833,6 +836,7 @@
 			return -1;
 	}
 
+	getrusage(RUSAGE_SELF, &ru3);
 	if (new_changed || cur_changed) {
 		/* if we're going to check cur/ dir our current logic requires
 		   that new/ dir is checked as well. it's a good idea anyway. */
@@ -846,6 +850,7 @@
 			if (++count >= MAILDIR_SCAN_DIR_MAX_COUNT)
 				final = TRUE;
 		}
+		getrusage(RUSAGE_SELF, &ru4);
 		if (ret < 0)
 			return -1;
 
@@ -853,11 +858,14 @@
 			if (maildir_scan_dir(ctx, FALSE, TRUE) < 0)
 				return -1;
 		}
+		getrusage(RUSAGE_SELF, &ru5);
 
 		maildir_sync_update_next_uid(ctx->mbox);
 
 		/* finish uidlist syncing, but keep it still locked */
 		maildir_uidlist_sync_finish(ctx->uidlist_sync_ctx);
+	} else {
+		ru4 = ru5 = ru3;
 	}
 
 	if (!ctx->locked) {
@@ -884,6 +892,7 @@
 		i_assert(maildir_uidlist_is_locked(ctx->mbox->uidlist) ||
 			 lock_failure);
 	}
+	getrusage(RUSAGE_SELF, &ru6);
 
 	if (find_uid != NULL && *find_uid != 0) {
 		ret = maildir_uidlist_lookup_nosync(ctx->mbox->uidlist,
@@ -899,7 +908,12 @@
 		}
 	}
 
-	return maildir_uidlist_sync_deinit(&ctx->uidlist_sync_ctx, TRUE);
+	ret = maildir_uidlist_sync_deinit(&ctx->uidlist_sync_ctx, TRUE);
+	getrusage(RUSAGE_SELF, &ru7);
+	i_debug("maildir sync volcs=%ld,%ld,%ld,%ld,%ld,%ld,%ld",
+		ru1.ru_nvcsw, ru2.ru_nvcsw, ru3.ru_nvcsw, ru4.ru_nvcsw,
+		ru5.ru_nvcsw, ru6.ru_nvcsw, ru7.ru_nvcsw);
+	return ret;
 }
 
 int maildir_storage_sync_force(struct maildir_mailbox *mbox, uint32_t uid)
